﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace VoiceConsole {
	/// <summary>
	/// Произносим текст (через соединение с VoiceServer)
	/// </summary>
	public static class Voice {
		#region Переменные
		private static string _lastText;
		private static VoiceManager _vm;
		#endregion
		
		#region Конструктор
		// Конструктор
		static Voice() {
			if (Voice._vm == null) {
				Voice._vm = new VoiceManager();
			}
		}
		#endregion

		#region Публичное
		/// <summary>
		/// Говорим время
		/// </summary>
		public static void SpeakTime() {
			Voice.SpeakWait("Время " + DateTime.Now.Hour.ToString("00") + ":" + DateTime.Now.Minute.ToString("00"));
		}

		/// <summary>
		/// Говорим и ждем
		/// </summary>
		/// <param name="text"></param>
		public static void SpeakWait(string text) {
			Voice.Speak(text);
			while (Voice.IsSpeaking()) { Application.DoEvents(); }
		}

		/// <summary>
		/// Говорим текст без сравнения с последним текстом
		/// </summary>
		/// <param name="text"></param>
		public static void SpeakNoCheck(string text) {
			Voice._lastText = "";
			Voice.Speak(text);
		}
				
		/// <summary>
		/// Произносим текст
		/// </summary>
		/// <param name="text"></param>
		public static void Speak(string text) {
			if (string.IsNullOrEmpty(text)) { return; }

			lock (Voice._vm) {
				if (Voice._lastText == text) { return; }
				Voice._lastText = text;
				Voice._vm.Speak(text);
			}
		}

		/// <summary>
		/// Очищаем очередь
		/// </summary>
		public static void Clear() {
			lock (Voice._vm) {
				Voice._vm.Clear();
			}
		}

		/// <summary>
		/// Остановить текст
		/// </summary>
		public static void Stop() {
			lock (Voice._vm) {
				if (Voice._vm.IsSpeaking())
					Voice._vm.Stop();
			}
			Application.DoEvents();
		}

		/// <summary>
		/// Возвращаем размер очереди
		/// </summary>
		/// <returns></returns>
		public static int GetCount() {
			lock (Voice._vm) {
				return Voice._vm.GetCount();
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public static bool IsSpeaking() {
			lock (Voice._vm) {
				return Voice._vm.IsSpeaking();
			}
		}
		#endregion
	}
}