﻿using System;
using System.Collections.Generic;
using System.Threading;
using SpeechLib;

namespace VoiceConsole {
	/// <summary>
	/// 
	/// </summary>
	public class VoiceManager {
		#region Переменные
		private SpVoice _sp;
		private bool _isSpeaking;
		private List<string> _texts = new List<string>();
		private Thread _threadVoice;
		#endregion

		#region Конструктор
		/// <summary>
		/// Конструктор
		/// </summary>
		public VoiceManager() {
			this._startThreadVoice();
		}
		#endregion

		#region Публичное
		/// <summary>
		/// Добавляем текст в очередь для озвучивания
		/// </summary>
		/// <param name="text"></param>
		/// <returns></returns>
		public void Speak(string text) {
			lock (this._texts) {
				this._texts.Add(text);
			}
		}

		/// <summary>
		/// Очищаем очередь на озвучивание
		/// </summary>
		/// <returns></returns>
		public void Clear() {
			if (this._texts != null) { lock (this._texts) { this._texts.Clear(); } }
		}

		/// <summary>
		/// Приостанавливаем озвучивание
		/// </summary>
		public void Pause() {
			if (this._sp != null) { this._sp.Pause(); }
		}

		/// <summary>
		/// Продолжаем озвучивание
		/// </summary>
		public void Resume() {
			if (this._sp != null) { this._sp.Resume(); }
		}

		/// <summary>
		/// Останавливаем озвучивание и прерываем текущий текст
		/// </summary>
		public void Stop() {
			this._startThreadVoice();
		}

		/// <summary>
		/// Есть ли что говорить или говорит в настоящий момент
		/// </summary>
		public bool IsSpeaking() {
			return (this._isSpeaking || this._texts.Count > 0);
		}

		/// <summary>
		/// Возвращаем размер очереди на озвучивание
		/// </summary>
		/// <returns></returns>
		public int GetCount() {
			return this._texts.Count;
		}
		#endregion

		#region Утилиты
		private void _startThreadVoice() {
			if (this._sp != null) {
				this._sp.Pause();
			}
			if (this._threadVoice != null) { this._threadVoice.Abort(); }
			this._sp = null;
			this._texts.Clear();

			this._threadVoice = new Thread(this._threadFunc);
			this._threadVoice.IsBackground = true;
			this._threadVoice.Start();
		}
	
		private void _threadFunc() {
			while (true) {
				this._isSpeaking = false;
				string text = "";
				try {
					if (this._sp == null) {
						this._sp = new SpVoice();
					}
					if (this._texts.Count > 0) {
						lock (this._texts) {
							text = this._texts[0];
							this._texts.RemoveAt(0);
						}
						//File.WriteTextUTF8(File.ApplicationPath() + @"Temp\" + Core.GetCompactDateTime() + ".txt", text);
						this._isSpeaking = true;
						this._sp.Speak(text);
						this._isSpeaking = false;
						text = "";
					}
				}
				catch (ThreadAbortException) { }
				catch (Exception) {
					this._sp = null;
					if (!string.IsNullOrEmpty(text)) { this.Speak(text); }
				}
				Thread.Sleep(1);
			}
		}
		#endregion
	}
}